#
# Nato (ICAO) Alphabet
#

from ..alphabet import Alphabet, alphabet_mgr

nato_alphabet = [
    'Alpha',
    'Bravo',
    'Charlie',
    'Delta',
    'Echo',
    'Foxtrot',
    'Golf',
    'Hotel',
    'India',
    'Juliett',
    'Kilo',
    'Lima',
    'Mike',
    'November',
    'Oscar',
    'Papa',
    'Quebec',
    'Romeo',
    'Sierra',
    'Tango',
    'Uniform',
    'Victor',
    'Whiskey',
    'Xray',
    'Yankee',
    'Zulu'
]

class Nato(Alphabet):

    name="nato"
    description="NATO (ICAO) Phonetic Alphabet"
    def_nwords=5

    def __init__(self):
        pass

    def convert(self, number: int):
        words = list()

        sz = len(nato_alphabet)

        while True:
            remainder = number % sz
            words.append(nato_alphabet[remainder])
            number = number // sz

            if number == 0:
                break

        return words

alphabet_mgr.add(Nato.name, Nato())






import hashlib
import io
import sys
from typing import Any

__version__ = '0.0.1'
default_alphabet = 'rfc1751'



from .alphabets import nato, rfc1751, eng1
from .alphabet import alphabet_mgr, Alphabet


class PinkHash():
    
    alphabet: Alphabet = None
    nbytes: int
    nwords: int


    def __init__(self, alphabet_name: str=None, nbytes: int = 8, nwords: int | None = None, donothash=False):
        
        self.nbytes = nbytes
        # donothash - if data is already bytes of hash
        self.donothash = donothash
        self.alphabet = alphabet_mgr.get_alphabet(alphabet_name or default_alphabet)
        self.nwords = nwords or self.alphabet.def_nwords

    def any2bytes(self, data: Any):
        # do not do anything if already bytes
        if isinstance(data, bytes):
            if self.donothash:
                return data
            hash_object = hashlib.sha1()
            hash_object.update(data)
            data = hash_object.digest()
            # print(hash_object.hexdigest(), file=sys.stderr)
            return data

        # if anything else
        if isinstance(data, str):
            # convert data to hash
            hash_object = hashlib.sha256()
            hash_object.update(data.encode('utf-8'))
            data = hash_object.digest()
            return data

        elif isinstance(data, io.IOBase):
            # read data from file
            data = data.read()
            
        else:
            print(f"Data type {type(data)} not supported yet")


    def convert(self, data: Any):
        data = self.any2bytes(data)
        last_bytes = data[-self.nbytes:]
        number = int.from_bytes(last_bytes, byteorder='big')  
        
        pinkhash = self.alphabet.convert(number = number)[:self.nwords]
        return ' '.join(pinkhash)

    def __repr__(self):
        return 'Pink Elephant'



import argparse
import sys

from . import __version__
from . import PinkHash
from .alphabet import alphabet_mgr

def get_args():
    parser = argparse.ArgumentParser(
        description=f'PinkHash version {__version__}. Convert data to memorable strings',
        formatter_class=argparse.RawTextHelpFormatter)

    parser.epilog = 'Available alphabets (-a) are:\n'
    for ab in alphabet_mgr.alphabets.values():
        parser.epilog += f'  {ab.name}: {ab.description}\n'

    parser.add_argument('-a', '--alphabet', metavar='Alphabet', type=str, 
                        help=f'One of: {" ".join(alphabet_mgr.alphabets.keys())}')
    parser.add_argument('-w', '--words', metavar='N_WORDS', type=int, default=3, help='Pink-hash lenght in words')

    g = parser.add_argument_group('Hashing inbound data')

    args = parser.parse_args()
    return args

def main():
    args = get_args()

    pink = PinkHash(alphabet_name=args.alphabet, nwords=args.words)

    data = sys.stdin.buffer.read()

    r = pink.convert(data)
    print(r)

    print()



#
# wordnet/nltk dictionary
#

from ..alphabet import Alphabet, alphabet_mgr
from pathlib import Path
import os

script_dir = os.path.dirname(os.path.abspath(__file__))



class Eng1(Alphabet):

    name="eng1"
    description=f"wordnet-based english alphabet (adj+noun)"
    def_nwords=5


    def __init__(self):
        self.nouns = None
        self.adjectives = None

    def load_wordlists(self):
        self.nouns = (Path(script_dir) / Path('eng1-nouns.txt')).read_text().splitlines()
        self.adjectives = (Path(script_dir) / Path('eng1-adjective.txt')).read_text().splitlines()


    def convert(self, number: int):
        words = list()

        if self.nouns is None:
            # load wordlists on first conversion
            self.load_wordlists()

        remainder = number % len(self.adjectives)
        words.append(self.adjectives[remainder])
        number = number // len(self.adjectives)

        remainder = number % len(self.nouns)
        words.append(self.nouns[remainder])

        return words

alphabet_mgr.add(Eng1.name, Eng1())







# abstract class for any alphabet such as Nato
class Alphabet():
    name = None
    description = "Abstract class for any alphabets"
    def_nwords=3

    def __init__(self):
        pass

class AlphabetManager():
    alphabets = dict()
    def __init__(self):
        pass

    def add(self, name: str, alphabet: Alphabet):
        self.alphabets[name] = alphabet

    def __repr__(self):
        return f'Alphabet Manager({self.alphabets.keys()})'
    
    def get_alphabet(self, name: str):
        return self.alphabets[name]


alphabet_mgr = AlphabetManager()



# generate dictionaries for eng1

import nltk
from nltk.corpus import wordnet as wn
from pathlib import Path

def clean_up(words_set: set):
    bad_characters = set("0123456789_-\'.")

    def has_bad_characters(word):
        return any(char in bad_characters for char in word)

    return {word for word in words_set if not has_bad_characters(word)}



def get_words(pos):
    # Get all noun synsets
    synsets = wn.all_synsets(pos=pos)

    # Extract words from synsets
    lemmas = set()
    for synset in synsets:
        for lemma in synset.lemmas():
            lemmas.add(lemma.name().lower())  # Add the noun to the set to avoid duplicates

    clean_up(lemmas)

    # Convert to a sorted list if desired
    word_list = sorted(clean_up(lemmas))
    return word_list

def main():
    try:
        nltk.data.find('corpora/wordnet.zip')  # Check for the WordNet data
    except LookupError:
        print("WordNet not found. Downloading...")
        nltk.download('wordnet')

    nouns_list = get_words(wn.NOUN)
    # save list to .txt file
    Path('eng1-nouns.txt').write_text('\n'.join(nouns_list))

    adj_list = get_words(wn.ADJ)
    # save list to .txt file
    Path('eng1-adjective.txt').write_text('\n'.join(adj_list))

if __name__ == "__main__":
    main()